{################################################################### LIB.INC }
{ Purpose...: General purpose library functions and global types/constants
{             This file must be included as the first include file
{ DepandsOn.: nothing
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{******************************************************** global definitions }
TYPE g_tpBoolean   = ^BOOLEAN;                   { boolean pointer           }
TYPE g_tpByte      = ^BYTE;                      { byte pointer              }
TYPE g_tpChar      = ^CHAR;                      { char pointer              }
TYPE g_tpInt       = ^INTEGER;                   { integer pointer           }
TYPE g_tst         = STRING[255];                { string type               }
TYPE g_tpst        = ^g_tst;                     { pointer to string type    }
TYPE g_tstLine     = STRING[100];                { string for in/out lines   }
TYPE g_tpstLine    = ^g_tstLine;                 { pointer to in/out lines   }
TYPE g_tstFile     = STRING[12];                 { string for filenames      }
TYPE g_tpstFile    = ^g_tStFile;                 { pointer to filename       }
TYPE g_tstHexAddr  = STRING[4];                  { string for hex addresses  }
TYPE g_tpstHexAddr = ^g_tstHexAddr;              { pointer to HEX address    }
TYPE g_tstHexData  = STRING[2];                  { string for hex data       }

CONST g_cESC   : CHAR      = ^[;                 { escape key                }
CONST g_cLF    : CHAR      = ^J;                 { line feed character       }
CONST g_cCR    : CHAR      = ^M;                 { cursor return character   }
CONST g_cFF    : CHAR      = ^L;                 { form feed (for printer)   }
CONST g_cTAB   : CHAR      = ^I;                 { TAB key                   }
CONST g_cBS    : CHAR      = ^H;                 { backspace key             }
CONST g_cEOF   : CHAR      = ^Z;                 { EOF character             }
CONST g_stCRLF : STRING[2] = ^M^J;               { return & linefeed         }
CONST g_cCTRLC : CHAR      = ^C;                 { CTRL-C                    }
CONST g_cCTRLE : CHAR      = ^E;                 { CTRL-E                    }
CONST g_cCTRLP : CHAR      = ^P;                 { CTRL-P                    }
CONST g_cCTRLR : CHAR      = ^R;                 { CTRL-R                    }
CONST g_cCTRLX : CHAR      = ^X;                 { CTRL-X                    }

CONST g_bINT_EQ: BYTE = 0;                       { int compare: equal        }
CONST g_bINT_LT: BYTE = 1;                       {              less than    }
CONST g_bINT_GT: BYTE = 2;                       {              greater than }

{************************************************************ implementation }
{======================================================= Lib_PrintFileName() }
{ Purpose...: Print a filename at current cursor position
{ Input.....: i_bNbr: Count of file. Used to do a line break after five names
{             i_iBuf: Buffer address where filename is stored
{============================================================================}
PROCEDURE Lib_PrintFileName(i_bNbr: BYTE; i_iBuf: INTEGER);

  VAR bNdx: BYTE;
  VAR pbBuf: ^BYTE;

BEGIN
  pbBuf := PTR(i_iBuf);

  {----------------------------- print name, 8 chars, end-padded with spaces }
  FOR bNdx := 0 TO 8 DO BEGIN

    IF (pbBuf^ >= 32) THEN BEGIN
      WRITE(CHR(pbBuf^ AND $7F));
    END;

    pbBuf := PTR(ORD(pbBuf) + 1);
  END;

  WRITE('.');

  {---------------------------- print extension, also end-padded with spaces }
  FOR bNdx := 9 TO 11 DO BEGIN

    IF (pbBuf^ >= 32) THEN BEGIN
      WRITE(CHR(pbBuf^ AND $7F));
    END;

    pbBuf := PTR(ORD(pbBuf) + 1);
  END;

  {--------------- do a linefeed if five names printed, else print separator }
  IF (i_bNbr MOD 5 <> 0) THEN BEGIN
    WRITE(' | ');
  END ELSE BEGIN
    WRITELN;
  END;
END;
{=================================================== eoP Lib_PrintFileName() }

{======================================================== Lib_GetDirectory() }
{ Purpose....: Read disk directory and display at cursor position
{ Input......: i_stDirMask: Mask to apply, '?' in any position means match all
{============================================================================}
PROCEDURE Lib_PrintDirectory(i_stDirMask: g_tstFile);

  {------------------------------------------ define CP/M file control block }
  TYPE rFcb = RECORD
                bDrive: BYTE;                    { drive, 0=current, 1=A..   }
                acName: ARRAY[0..7] OF CHAR;     { filename (7-bit ASCII)    }
                acExt:  ARRAY[0..2] OF CHAR;     { extension (7-bit ASCII)   }
                bEx:    BYTE;
                bS1:    BYTE;                    { reserved                  }
                bS2:    BYTE;                    { reserved                  }
                bRc:    BYTE;
                bAl:    BYTE;                    { file allocation on disk   }
                bCr:    BYTE;                    { current record within ext }
                wRn:    INTEGER;                 { random access record nbr  }
              END;

  VAR sFcb: rFcb;
  VAR abDma: ARRAY[0..127] OF BYTE;
  VAR bFileCnt: BYTE;
  VAR bCnt: BYTE;
  VAR bBdosResult: BYTE;

BEGIN
  bFileCnt := 1;

  {------------------------------------------------ setup file control block }
  sFcb.bDrive := 0;                              { use default drive         }

  FOR bCnt := 0 TO 7 DO BEGIN
    sFcb.acName[bCnt] := i_stDirMask[bCnt + 1];
  END;

  FOR bCnt := 0 TO 2 DO BEGIN
    sFcb.acExt[bCnt] := i_stDirMask[bCnt + 8 + 1];
  END;

  sFcb.bEx := 0;                                 { all others unused         }
  sFcb.bS1 := 0;
  sFcb.bS2 := 0;
  sFcb.bRc := 0;
  sFcb.bAl := 0;
  sFcb.bCr := 0;
  sFcb.wRn := 0;

  {---------------------------------------------------- set CP/M DMA address }
  bBdosResult := BDOS(26, ADDR(abDma));

  {-------------------------------------------------------------- search 1st }
  bBdosResult := BDOS(17, ADDR(sFcb));

  IF (bBdosResult <> $FF) THEN BEGIN
    Lib_PrintFileName(bFileCnt, ADDR(abDma[bBdosResult * 32]));
    bFileCnt := bFileCnt + 1;
  END;

  {-------------------------------------------------- serach next until done }
  WHILE (bBdosResult <> $FF) DO BEGIN
    bBdosResult := BDOS(18, ADDR(sFcb));

    IF (bBdosResult <> $FF) THEN BEGIN
      Lib_PrintFileName(bFileCnt, ADDR(abDma[bBdosResult * 32]));
      bFileCnt := bFileCnt + 1;
    END ELSE BEGIN
      IF (bFileCnt >= 1) THEN BEGIN
        WRITE(CHR(8), CHR(8), ' ');
      END;
    END;
  END;
END;
{================================================== eoF Lib_PrintDirectory() }

{=========================================================== Lib_IsHexChar() }
{ Purpose...: IF ch IN ['A'..'F'] does not work, type mismatch
{ Input.....: i_cCh: Character to check
{ Result....: TRUE: character is HEX, FALSE= it is not
{============================================================================}
FUNCTION Lib_IsHexChar(i_cCh: CHAR): BOOLEAN;

  VAR fDigit: BOOLEAN;
  VAR fLetter: BOOLEAN;

BEGIN
  fDigit := ((i_cCh >= '0') AND (i_cCh <= '9'));
  fLetter := ((i_cCh >= 'A') AND (i_cCh <= 'F'));

  Lib_IsHexChar := (fDigit OR fLetter);
END;
{======================================================= eoF Lib_IsHexChar() }

{============================================================== Lib_CmpInt() }
{ Purpose...: Integer values > 32768 cannot be compared using <, =, >
{ Input.....: i_iVal1: 1st value
{             i_iVal2L 2nd value
{ Result....: g_bINT_EQ: the values are equal
{             g_bINT_LT: i_iVal1 is less than i_iVal2
{             g_bINT_GT: i_iVal1 is greater than i_iVal2
{============================================================================}
FUNCTION Lib_CmpInt(i_iVal1: INTEGER; i_iVal2: INTEGER):BYTE;

  VAR iHi1: INTEGER;
  VAR iLo1: INTEGER;
  VAR iHi2: INTEGER;
  VAR iLo2: INTEGER;
  VAR bFnResult: BYTE;

BEGIN
  iHi1 := HI(i_iVal1);
  iLo1 := LO(i_iVal1);
  iHi2 := HI(i_iVal2);
  iLo2 := LO(i_iVal2);

  IF (i_iVal1 = i_iVal2) THEN BEGIN
    bFnResult := g_bINT_EQ;
  END ELSE BEGIN

    IF (iHi1 > iHi2) THEN BEGIN
      bFnResult := g_bINT_GT;
    END ELSE BEGIN

      IF (iHi1 < iHi2) THEN BEGIN
        bFnResult := g_bINT_LT;
      END ELSE BEGIN

        IF (iLo1 > iLo2) THEN BEGIN
          bFnResult := g_bINT_GT;
        END ELSE BEGIN

          IF (iLo1 < iLo2) THEN BEGIN
            bFnResult := g_bINT_LT;
          END;
        END;
      END;
    END;
  END;

  Lib_CmpInt := bFnResult;
END;
{========================================================== eoF Lib_CmpInt() }

{============================================================ Lib_WriteInt() }
{ Purpose...: It is not possible to WRITE an integer > 32768 as such.
{             It will always be printed as negative value
{ Input.....: i_iVal...: Integer value to WRITE
{             i_bDigits: Number of digits to write. The required number is
{                        always printed, if i_bDigits is greater, spaces
{                        are inserted in front of number.
{============================================================================}
PROCEDURE Lib_WriteInt(i_iVal: INTEGER; i_bDigits: BYTE);

BEGIN

  IF (i_iVal < 0) THEN BEGIN
    WRITE(65536.0 + i_iVal:i_bDigits:0);
  END ELSE BEGIN
    WRITE(0.0 + i_iVal:i_bDigits:0);
  END;
END;
{======================================================== eoP Lib_WriteInt() }

{=========================================================== Lib_CnvUpCase() }
{ Purpose...: Convert string to upper case
{ Input.....: i_stString: string to convert
{ Result....: Uppercase string
{============================================================================}
FUNCTION Lib_CnvUpCase(i_stString: g_tst): g_tst;

  VAR bLen: BYTE;
  VAR bCnt: BYTE;

BEGIN
  bCnt := 1;
  bLen := LENGTH(i_stString);

  WHILE(bCnt <= bLen) DO BEGIN
    i_stString[bCnt] := UPCASE(i_stString[bCnt]);
    bCnt := bCnt + 1;
  END;

  Lib_CnvUpCase := i_stString;
END;
{======================================================= eoF Lib_CnvUpCase() }

{========================================================= Lib_CnvHexToBin() }
{ Purpose...: Convert HEX string to binary
{ Input.....: i_stHex: HEX string to convert (max 4 chars)
{ Result....: Numeric value
{============================================================================}
FUNCTION Lib_CnvHexToBin(i_stHex: g_tstHexAddr): INTEGER;

  VAR iFnResult: INTEGER;
  VAR bLen: BYTE;
  VAR bNdx: BYTE;
  VAR bValue: BYTE;

BEGIN
  iFnResult := 0;
  bNdx := 1;
  bLen := LENGTH(i_stHex);

  WHILE (bNdx <= bLen) DO BEGIN
    iFnResult := iFnResult * 16;
    bValue := ORD(UPCASE(i_stHex[bNdx]));
    bValue := bValue - ORD('0');

    IF (bValue > 9) THEN BEGIN
      bValue := bValue - 7;
    END;

    iFnResult := iFnResult + bValue;
    bNdx := bNdx + 1;
  END;

  Lib_CnvHexToBin := iFnResult;
END;
{===================================================== eoF Lib_CnvHexToBin() }

{========================================================= Lib_CnvBinToHex() }
{ Purpose...: Convert binary to HEX string
{ Input.....: i_iBin: Binary number to convert
{             i_bLen: Length of resulting string
{ Result....: HEX string, left-padded with '0'
{============================================================================}
FUNCTION Lib_CnvBinToHex(i_iBin: INTEGER; i_bLen: BYTE): g_tstHexAddr;

  VAR stFnResult: g_tstHexAddr;
  VAR iDiv:       INTEGER;
  VAR bDigit:     BYTE;
  VAR bNdx:       BYTE;

BEGIN
  stFnResult := '0000';
  iDiv := $1000;
  bNdx := 1;

  WHILE (i_iBin <> 0) DO BEGIN
    bDigit := (i_iBin DIV iDiv);
    i_iBin := i_iBin - (bDigit * iDiv);
    iDiv := (iDiv SHR 4);
    IF (bDigit >9) THEN bDigit := bDigit + 7;
    stFnResult[bNdx] := CHR(bDigit + 48);
    bNdx := bNdx + 1;
  END;

  IF (i_bLen > 4) THEN i_bLen := 4;
  IF (i_bLen = 0) THEN i_bLen := 1;

  stFnResult := COPY(stFnResult, 5-i_bLen, i_bLen);
  Lib_CnvBinToHex := stFnResult;
END;
{===================================================== eoF Lib_CnvBinToHex() }

{============================================================== Lib_GetKey() }
{ Purpose...: Get key from keyboard
{ Input.....: i_stKeys: Keys to recognize
{             i_fWait.: Wait for key
{ Result....: Keystroke entered, 0 if none
{============================================================================}
FUNCTION Lib_GetKey(i_stKeys: g_tst; i_fWait: BOOLEAN): CHAR;

  VAR cFnResult: CHAR;
  VAR cKey: CHAR;
  VAR fKeyValid: BOOLEAN;

BEGIN
  fKeyValid := FALSE;

  {--------------------------- if not wait, read keyboard data port directly }
  IF (i_fWait = FALSE) THEN BEGIN
    cKey := CHR(PORT[5]);
    fKeyValid := (POS(cKey, i_stKeys) <> 0);
  END

  {-------------------------------------------- wait for key, use KEYPRESSED }
  ELSE BEGIN
    REPEAT
      IF (KEYPRESSED) THEN BEGIN
        cKey := CHR(PORT[5]);
        fKeyValid := (POS(cKey, i_stKeys) <> 0);
      END;
    UNTIL ((fKeyValid = TRUE) OR (i_fWait = FALSE));
  END;

  IF (fKeyValid = FALSE) THEN BEGIN
    cKey := CHR(0);
  END;

  Lib_GetKey := cKey;
END;
{========================================================== eoF Lib_GetKey() }

{============================================================== Lib_GetNum() }
{ Purpose...: Get hex or dec value from keyboard
{ Input.....: i_piVal..: Where to store integer result
{             i_pstVal.: Where to store ASCII result
{             i_bLen...: Number of digits to read (8 max)
{             i_fHex...: TRUE= read HEX characters, FALSE= read DEC
{ Result....: TRUE= success, FALSE= aborted
{============================================================================}
FUNCTION Lib_GetNum(i_piVal: g_tpInt; i_pstVal: g_tpstHexAddr; i_bLen: BYTE;
                    i_fHex: BOOLEAN): BOOLEAN;

  CONST stNUMKEYS = '0123456789';
  CONST stHEXKEYS = 'AaBbCcDdEeFf';

  TYPE tstNum  = STRING[8];
  TYPE tstKeys = STRING[30];

  VAR fFnResult  : BOOLEAN;
  VAR cKey       : CHAR;
  VAR bLen       : BYTE;
  VAR stNum      : tstNum;
  VAR stKeys     : tstKeys;
  VAR iValResult : INTEGER;
  VAR fDone      : BOOLEAN;
  VAR fAbort     : BOOLEAN;

BEGIN
  fFnResult := TRUE;
  fAbort := FALSE;
  bLen := 0;
  stKeys := stNUMKEYS + g_cBS + g_cESC;
  stNum := '';

  IF (i_bLen > 8) THEN i_bLen := 8;

  {----------------------------------- define valid keys due to HEX/DEC mode }
  IF (i_fHex = TRUE) THEN BEGIN
    stKeys := stKeys + stHEXKEYS;
  END ELSE BEGIN
    stKeys := stKeys + g_cCR;
  END;

  {----------------------------------------------------- loop number of keys }
  WHILE ((bLen < i_bLen) AND (fAbort = FALSE)) DO BEGIN
    fDone := FALSE;
    cKey := UPCASE(Lib_GetKey(stKeys, TRUE));

    {------------------------------------------------- process backspace key }
    IF (cKey = g_cBS) THEN BEGIN
      fDone := TRUE;

      IF (bLen > 0) THEN BEGIN
        stNum[bLen] := '0';
        bLen := bLen - 1;
        WRITE(g_cBS + ' ' + g_cBS);
      END;
    END;

    {----------------------------------- enter key is valid in dec mode only }
    IF ((fDone = FALSE) AND (cKey = g_cCR)) THEN BEGIN
      fDone := TRUE;

      IF (bLen >= 1) THEN BEGIN
        fAbort := TRUE;
      END;
    END;

    {------------------------------------------------ ESC was pressed, abort }
    IF ((fDone = FALSE) AND (cKey = g_cESC)) THEN BEGIN
      fDone := TRUE;
      fAbort := TRUE;
      fFnResult := FALSE;
    END;

    {---------------------------------------------- process HEX and DEC keys }
    IF (fDone = FALSE) THEN BEGIN
      bLen := bLen + 1;
      stNum[bLen] := cKey;
      WRITE(cKey);
    END;
  END;

  {---------------------------------------------------------- process result }
  IF (fFnResult = TRUE) THEN BEGIN
    stNum[0] := CHR(bLen);

    IF (i_piVal <> NIL) THEN BEGIN
      IF (i_fHex = TRUE) THEN BEGIN
        i_piVal^ := Lib_CnvHexToBin(stNum);
      END ELSE BEGIN
        VAL(stNum, i_piVal^, iValResult);
      END;
    END;

    IF (i_pstVal <> NIL) THEN BEGIN
      i_pstVal^ := stNum;
    END;
  END;

  Lib_GetNum := fFnResult;
END;
{========================================================== eoF Lib_GetNum() }

{=============================================================== Lib_Delay() }
{ Purpose...: Perform a simple delay loop
{ Input.....: i_iDelay: Loop counter
{============================================================================}
PROCEDURE Lib_Delay(i_iDelay: INTEGER);

BEGIN
  WHILE(i_iDelay <> 0) DO BEGIN
    i_iDelay := i_iDelay - 1;
  END;
END;
{=========================================================== eoF Lib_Delay() }

{############################################################### EOF LIB.INC }