{############################################################## TM990MNU.INC }
{ Purpose...: TM990 menu and file selection
{ DependsOn.: LIB.INC, LIBPRT.INC, TM990FIL.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{----- line number where selection prompt is printed (used for file listing) }
CONST m_bPROMPT_LINE = 15;

{----------------------------------------------- length of menu border lines }
CONST m_bBORDER_LENGTH = 32;

{************************************************************ implementation }
{========================================================= Menu_SelectFile() }
{ Purpose...: Select a file
{ Input.....: i_bOperation: m_bOPERATION_XXX
{ Result....: TRUE= File entered/selected, FALSE= aborted
{============================================================================}
FUNCTION Menu_SelectFile(i_bOperation: BYTE): BOOLEAN;

  CONST bMODE_TRANSMIT = 1;
  CONST bMODE_RECEIVE  = 2;
  CONST bMODE_VIEW     = 3;

  VAR stFileExt: g_tStFile;
  VAR bCnt:      BYTE;
  VAR bMode:     BYTE;

BEGIN
  bMode := bMODE_RECEIVE;
  stFileExt := '';
  WRITE(g_cCR, 'Reading directory...');

  CASE i_bOperation OF
    g_bOPERATION_RECEIVE_MEM:  BEGIN
                                 stFileExt := '.MEM';
                               END;
    g_bOPERATION_TRANSMIT_MEM: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.MEM';
                               END;
    g_bOPERATION_TRANSMIT_ASM: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.ASM';
                               END;
    g_bOPERATION_RECEIVE_BAS:  BEGIN
                                 stFileExt := '.BAS';
                               END;
    g_bOPERATION_TRANSMIT_BAS: BEGIN
                                 bMode := bMODE_TRANSMIT;
                                 stFileExt := '.BAS';
                               END;
    g_bOPERATION_VIEW_TXT:     BEGIN
                                 bMode := bMODE_VIEW;
                                 stFileExt := '.???';
                               END;
  END;

  {------------------------------- display directory of selected file types }
  WRITELN; WRITELN;
  IF (stFileExt = '.MEM') THEN WRITE('MemImage');
  IF (stFileExt = '.ASM') THEN WRITE('AsmSource');
  IF (stFileExt = '.BAS') THEN WRITE('Basic');
  IF (stFileExt = '.???') THEN WRITE('Text');
  WRITELN(' files on disk');
  Lib_PrintDirectory('????????' + COPY(stFileExt,2,4));

  {----------------------------------------------------------- get filename }
  GOTOXY(1, m_bPROMPT_LINE);
  CLREOL;

  IF (stFileExt <> '.???') THEN BEGIN
    WRITE(g_cCR, 'File (', stFileExt, ') to ');
  END ELSE BEGIN
    WRITE(g_cCR, 'File to ');
  END;

  CASE bMode OF
    bMODE_TRANSMIT: WRITE('transmit ');
    bMODE_RECEIVE:  WRITE('receive ');
    bMODE_VIEW:     WRITE('view ');
  END;

  g_stFile := '';
  READ(g_stFile);
  WRITE(g_cCR);

  {-------------------------------- clear area occupied by directory listing }
  FOR bCnt := m_bPROMPT_LINE TO 23 DO BEGIN
    GOTOXY(1, bCnt);
    CLREOL;
  END;

  GOTOXY(1, m_bPROMPT_LINE);

  IF (stFileExt <> '.???') THEN BEGIN
    IF (LENGTH(g_stFile) >8) THEN BEGIN
      g_stFile := COPY(g_stFile, 1, 8);
    END;
  END;

  {-------------------------------------------------------- open/create file }
  IF (LENGTH(g_stFile) <> 0) THEN BEGIN

    IF (stFileExt <> '.???') THEN BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile + stFileExt);
    END ELSE BEGIN
      g_stFile := Lib_CnvUpCase(g_stFile);
    END;

    IF (File_OpenFile(i_bOperation) = FALSE) THEN BEGIN
      g_stFile := '';
    END;
  END;

  Menu_SelectFile := (LENGTH(g_stFile) <> 0);
END;
{===================================================== eoF Menu_SelectFile() }

{==================================================== Menu_SelectPrintMode() }
{ Purpose...: Select a print mode, send command to printer
{============================================================================}
PROCEDURE Menu_SelectPrintMode;

  TYPE tstMode    = STRING[10];

  VAR cKbd         : CHAR;
  VAR fDone        : BOOLEAN;
  VAR fDummy       : BOOLEAN;
  VAR stMode       : tstMode;
  VAR iLeftMargin  : INTEGER;
  VAR piLeftMargin : ^INTEGER;
  VAR bPageLength  : BYTE;
  VAR bPageLines   : BYTE;

BEGIN
  fDone := FALSE;
  stMode := '';
  piLeftMargin := PTR(ADDR(iLeftMargin));

  {---------------------------------------------------------------- get mode }
  WHILE (fDone = FALSE) DO BEGIN
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;
    WRITE('<I>nit  <P>ica  <E>lite  <C>ond  E<m>ph  <L>eft margin  <F>orm ');
    cKbd := UPCASE(Lib_GetKey('IiPpCcEeMmLlFf' + g_cESC + g_cCR, TRUE));
    GOTOXY(1, m_bPROMPT_LINE);
    CLREOL;

    LibPrt_Setup(1, m_bPROMPT_LINE, TRUE);

    CASE cKbd OF
      'I'  : stMode := g_cLibPrtMODE_INIT;
      'P'  : stMode := g_cLibPrtMODE_PICA;
      'E'  : stMode := g_cLibPrtMODE_ELITE;
      'C'  : stMode := g_cLibPrtMODE_COND;
      'M'  : stMode := g_cLibPrtMODE_EMPH;
      'L'  : stMode := g_cLibPrtMODE_LMARGIN;
    END;

    fDone := ((cKbd = g_cESC) OR (cKbd = g_cCR));

    IF (fDone = FALSE) THEN BEGIN

      {------------------------------------------- enter left printer margin }
      IF (cKbd = 'L') THEN BEGIN
        WRITE('Left margin: ');

        IF (Lib_GetNum(piLeftMargin, NIL, 3, FALSE) = TRUE) THEN BEGIN
          stMode := stMode + CHR(iLeftMargin);
        END ELSE BEGIN
          stMode := '';
        END;

        GOTOXY(1, m_bPROMPT_LINE);
        CLREOL;
      END;

      {------------------------------------------------- select paper format }
      IF (cKbd = 'F') THEN BEGIN
        WRITE('<A>4  <B>5  <C>ontinuous ');
        cKbd := UPCASE(Lib_GetKey('AaBbCc' + g_cESC + g_cCR, TRUE));
        bPageLength := 0;
        bPageLines := 0;

        IF (cKbd = 'A') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_A4;
          bPageLines := g_bLibPrtPAGE_LINES_A4;
        END;

        IF (cKbd = 'B') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_B5;
          bPageLines := g_bLibPrtPAGE_LINES_B5;
        END;

        IF (cKbd = 'C') THEN BEGIN
          bPageLength := g_bLibPrtPAGE_LENGTH_CONT OR $80;
          bPageLines := g_bLibPrtPAGE_LINES_CONT;
        END;

        IF (bPageLength <> 0) THEN BEGIN
          LibPrt_SetForm(bPageLength, bPageLines);
        END;
      END;

      IF (LENGTH(stMode) <> 0) THEN BEGIN
        fDummy := LibPrt_Print(CHR(0), g_cESC + stMode);
      END;
    END;
  END;

  LibPrt_Setup(0, 0, TRUE);
END;
{================================================ eoP Menu_SelectPrintMode() }

{============================================================= Menu_Border() }
{ Purpose...: Draw menu borders. Implemented to save space (hopefully)
{============================================================================}
PROCEDURE Menu_Border;

  VAR bCnt : BYTE;

BEGIN
  FOR bCnt := 0 TO m_bBORDER_LENGTH DO BEGIN
    WRITE('=');
  END;

  WRITELN;
END;
{=========================================================== eoP Menu_Border }

{==================================================================== Menu() }
{ Purpose...: Display menu
{ Input.....: i_piSelection: Where to store selection
{ Result....: TRUE= a selection was made, FALSE= exit was selected
{============================================================================}
FUNCTION Menu(i_piOperation: INTEGER): BOOLEAN;

  VAR cKbd       : CHAR;
  VAR bOperation : BYTE;
  VAR fDrawMenu  : BOOLEAN;
  VAR fSelectFile: BOOLEAN;
  VAR stFileExt  : g_tStFile;

BEGIN
  fDrawMenu := TRUE;
  bOperation := 0;

  WHILE (bOperation = 0) DO BEGIN

    IF (fDrawMenu = TRUE) THEN BEGIN
      fDrawMenu := FALSE;
      CLRSCR;
      Menu_Border;
      WRITELN(g_stPROGRAM, ' V', g_stVERSION);
      Menu_Border;

      WRITE('-1- Receive Unibug MEM file'  + g_stCRLF +
            '-2- Transmit Unibug MEM file' + g_stCRLF +
            '-3- Assemble Unibug ASM file' + g_stCRLF +
            '-4- Receive BASIC file'       + g_stCRLF +
            '-5- Transmit BASIC file'      + g_stCRLF +
            '-T- Terminal'                 + g_stCRLF +
            '-V- View text file'           + g_stCRLF +
            '-U- Usage notes'              + g_stCRLF +
            '-P- Setup printer'            + g_stCRLF +
            '-X- Exit'                     + g_stCRLF
           );

      Menu_Border;
    END;

    WRITE(g_cCR);
    CLREOL;
    WRITE('Selection ');
    cKbd := UPCASE(Lib_GetKey('12345TtVvUuPpXx', TRUE));
    fSelectFile := TRUE;

    CASE cKbd OF

      '1': bOperation := g_bOPERATION_RECEIVE_MEM;
      '2': bOperation := g_bOPERATION_TRANSMIT_MEM;
      '3': bOperation := g_bOPERATION_TRANSMIT_ASM;
      '4': bOperation := g_bOPERATION_RECEIVE_BAS;
      '5': bOperation := g_bOPERATION_TRANSMIT_BAS;

      'T': BEGIN
             bOperation := g_bOPERATION_TERMINAL;
             fSelectFile := FALSE;
           END;

      'V': bOperation := g_bOPERATION_VIEW_TXT;

      'U': BEGIN
             bOperation := g_bOPERATION_USAGE;
             fSelectFile := FALSE;
             g_stFile := g_stFILE_USAGE;

             IF (File_OpenFile(g_bOPERATION_VIEW_TXT) = FALSE) THEN BEGIN
               bOperation := 0;
               g_stFile := '';
             END;
           END;

      'P': BEGIN
             bOperation := 0;
             fSelectFile := FALSE;
             Menu_SelectPrintMode;
           END;

      'X': BEGIN
             bOperation := g_bOPERATION_EXIT;
             fSelectFile := FALSE;
           END;
    END;

    {---------------------------------------------- select file to work with }
    IF (fSelectFile = TRUE) THEN BEGIN

      IF (Menu_SelectFile(bOperation) = FALSE) THEN BEGIN
        bOperation := 0;
      END;
    END;
  END;

  MEM[i_piOperation] := bOperation;
  Menu := (bOperation <> g_bOPERATION_EXIT);
END;
{================================================================ eoF Menu() }

{########################################################### EOF TMSMENU.INC }
