{############################################################## TM990TRM.INC }
{ Purpose...: TM990 terminal
{ DependsOn.: LIB.INC, LIBSIO.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{*************************************************************** definitions }
CONST m_iTERM_BAUD_RATE:  INTEGER  = 300;        { baudrate                  }
CONST m_iTERM_TXDLY_CHAR: INTEGER  = 0;          { tx delay between chars    }
CONST m_iTERM_TXDLY_LINE: INTEGER  = 0;          { tx delay between lines    }

{************************************************************ implementation }
{============================================================ Term_InitSio() }
{ Purpose...: Init/deinit SIO
{ Input.....: i_fInit: TRUE= init, FALSE= de-init
{============================================================================}
PROCEDURE Term_InitSio(i_fInit: BOOLEAN);

BEGIN

  LibSio_Init(i_fInit, m_iTERM_BAUD_RATE, 'E', 7, 1,
              m_iTERM_TXDLY_CHAR,m_iTERM_TXDLY_LINE);
END;
{======================================================== eoF Term_InitSio() }

{============================================================ Term_Message() }
{ Purpose...: Print a message to screen
{ Input.....: i_stMsg.: Base message
{             i_fOnOff: TRUE= print ON or OFF
{             i_fStat.: TRUE= ON, FALSE= OFF
{============================================================================}
PROCEDURE Term_Message(i_stMsg: g_tStLine;
                       i_fOnOff: BOOLEAN; i_fStat: BOOLEAN);

BEGIN
  WRITELN;
  WRITE('-=[', i_stMsg);

  IF (i_fOnOff = TRUE) THEN BEGIN
    WRITE(' is ');

    IF (i_fStat) = TRUE THEN BEGIN
      WRITE('ON');
    END ELSE BEGIN
      WRITE('OFF');
    END;
  END;

  WRITELN(']=-');
END;
{======================================================== eoP Term_Message() }

{==================================================================== Term() }
{ Purpose...: A thumb terminal
{============================================================================}
PROCEDURE Term;

  CONST HEX_CHARS_PER_LINE  = 16;

  VAR fExit      : BOOLEAN;
  VAR fEcho      : BOOLEAN;
  VAR fPrinter   : BOOLEAN;
  VAR fSkip      : BOOLEAN;
  VAR fPrint     : BOOLEAN;
  VAR fRx        : BOOLEAN;
  VAR fHex       : BOOLEAN;
  VAR bHexCnt    : BYTE;
  VAR bSioResult : BYTE;
  VAR bRx        : BYTE;
  VAR pbRx       : g_tpByte;
  VAR bHex       : BYTE;
  VAR cKbd       : CHAR;
  VAR stRx       : g_tstLine;
  VAR stTx       : g_tstLine;

BEGIN
  fExit := FALSE;
  fEcho := FALSE;
  fSkip := FALSE;
  fPrinter := FALSE;
  fRx := FALSE;
  fHex := FALSE;
  bHexCnt := 0;
  pbRx := PTR(ADDR(bRx));
  stRx := '';
  stTx := '';
  bSioResult := g_bLibSioNOK;

  CLRSCR;
  WRITELN('Terminal: Unibug: Letter key, Basic: UNIT2<CR>');
  WRITE('Commands: <^E>cho | <^P>rint | HE<X> | <C>lear', g_stMSG_CANCEL);
  WRITELN;

  Term_InitSio(TRUE);

  {----------------------------------------------------- loop until canelled }
  WHILE (fExit = FALSE) DO BEGIN

    {-------------------------------------------- display received character }
    IF (LibSio_RxByte(pbRx, 0, FALSE) = g_bLibSioOK) THEN BEGIN
      bHex := bRx;
      bRx := (bRx AND $7F);
      stRx := '';

      {------------------------------------------------- write data as ASCII }
      IF (fHex = FALSE) THEN BEGIN

        {----------------------- TM990 sends a ACK ($06) between $0A and $0D }
        IF ((bRx <> 0) AND (bRx <> 6)) THEN BEGIN
          IF ((bRx <> $13) AND (bRx <> $14)) THEN BEGIN
            stRx := CONCAT(stRx, CHR(bRx));
          END;
        END;
      END

      {---------------------------------------- write data as HEX characters }
      ELSE BEGIN

        IF (fRx = FALSE) THEN BEGIN
          fRx := TRUE;
          bHexCnt := 0;
          stRx := CONCAT(stRx, g_stCRLF);
        END;

        stRx := CONCAT(stRx, '[', Lib_CnvBinToHex(bHex, 2), ']');
        bHexCnt := bHexCnt + 1;

        {----------------------------------------------- format display line }
        IF (bHexCnt = (HEX_CHARS_PER_LINE / 2)) THEN BEGIN
          stRx := CONCAT(stRx, '  ');

        END ELSE BEGIN

          IF ((bHexCnt <> 0) AND (bHexCnt MOD 4 = 0)) THEN BEGIN
            stRx := CONCAT(stRx, ' ');
          END;
        END;

        IF (bHexCnt = HEX_CHARS_PER_LINE) THEN BEGIN
          bHexCnt := 0;
          stRx := CONCAT(stRx, g_stCRLF);
        END;
      END;
    END;

    {----------------------------------------- send character from keyboard }
    IF (KEYPRESSED) THEN BEGIN
      READ(KBD, cKbd);
      cKbd := UPCASE(cKbd);
      stTx := '';
      fSkip := FALSE;

      {--------------------------------------------------- clear the screen }
      IF (cKbd = g_cCTRLC) THEN BEGIN
        CLRSCR;
        fSkip := TRUE;
      END;

      {-------------------------------------------- toggle HEX data display }
      IF (cKbd = g_cCTRLX) THEN BEGIN
        fHex := (NOT fHex);
        Term_Message('Hex mode', TRUE, fHex);
        bHexCnt := 0;
        fSkip := TRUE;
      END;

      {------------------------------------------- toggle local screen echo }
      IF (cKbd = g_cCTRLE) THEN BEGIN
        fEcho := (NOT fEcho);
        Term_Message('Echo', TRUE, fEcho);
        fSkip := TRUE;
      END;

      {------------------------------------------------ toggle printer echo }
      IF (cKbd = g_cCTRLP) THEN BEGIN
        fPrinter := NOT fPrinter;

        IF ((LibPrt_Ready = FALSE) AND (fPrinter = TRUE)) THEN BEGIN
          Term_Message('Printer not ready', FALSE, FALSE);
          fPrinter := FALSE;
        END ELSE BEGIN
          Term_Message('Printer', TRUE, fPrinter);
          LibPrt_Setup(0, 0, TRUE);
        END;

        fSkip := TRUE;
      END;

      {--------------------------------------------------- <esc> was pressed }
      IF (cKbd = g_cESC) THEN BEGIN
        WRITELN;
        fExit := TRUE;
        fSkip := TRUE;
      END;

      {---------------------------- process keyboard char if no function key }
      IF (fSkip = FALSE) THEN BEGIN

        {------------------------------------------------------- normal mode }
        IF ((fEcho = TRUE) AND (fHex = FALSE)) THEN BEGIN
          stTx := CONCAT(stTx, cKbd);
          IF (cKbd = g_cCR) THEN stTx := CONCAT(stTx, g_cLF);
        END;

        {---------------------------------------------------------- HEX mode }
        IF ((fEcho = TRUE) AND (fHex = TRUE)) THEN BEGIN
          IF (fRx = TRUE) THEN BEGIN
            fRx := FALSE;
            bHexCnt := 0;
            stTx := CONCAT(stTx, g_stCRLF);
          END;

          stTx := CONCAT(stTx, '{', Lib_CnvBinToHex(ORD(cKbd), 2), '}');
          bHexCnt := bHexCnt + 1;

          IF (bHexCnt = HEX_CHARS_PER_LINE) THEN BEGIN
            bHexCnt := 0;
            stTx := CONCAT(stTx, g_stCRLF);
          END;
        END;

        fExit := (LibSio_TxByte(ORD(cKbd)) <> g_bLibSioOK);
      END;
    END;

    {-------------------------------------------------------------- printing }
    fPrint := FALSE;

    IF (LENGTH(stRx) <> 0) THEN BEGIN
      WRITE(stRx);
      fPrint := TRUE;
    END;

    IF (LENGTH(stTx) <> 0) THEN BEGIN
      WRITE(stTx);
      fPrint := TRUE;
    END;

    IF ((fPrinter = TRUE) AND (fPrint = TRUE)) THEN BEGIN
      LibSio_SetTxHs(g_bLibSioTX_HS_OFF);

      IF (LENGTH(stRx) <> 0) THEN BEGIN
        fPrinter := LibPrt_Print(CHR(0), stRx);
      END;

      IF (LENGTH(stTx) <> 0) THEN BEGIN
        fPrinter := LibPrt_Print(CHR(0), stTx);
      END;

      WHILE ((fPrinter = TRUE) AND (LibPrt_Ready = FALSE)) DO BEGIN END;
      LibSio_SetTxHs(g_bLibSioTX_HS_ON);
    END;

    stRx := '';
    stTx := '';
  END;

  Term_InitSio(FALSE);
END;
{================================================================ eoP Term() }

{########################################################## EOF TM990TRM.INC }
