{############################################################## TM990VIW.INC }
{ Purpose...: TM990 include file: View text file
{ DependsOn.: LIB.INC
{ Compiler..: Turbo-Pascal V3.0
{ Target....: Kaypro II
{ LastEdit..: 13.12.2022
{############################################################################}

{*************************************************************** definitions }
CONST m_bMSG_ROW   = 1;
CONST m_bMSG_COL   = 24;
CONST m_bSCR_LINES = 22;

VAR m_stFile : g_tstFile;

{************************************************************ implementation }
{============================================================= View_Prompt() }
{ Purpose...: Print prompt message 
{ Input.....: i_stMsg: Prompt string
{=========================================================================== }
PROCEDURE View_Prompt(i_stMsg: g_tst);

BEGIN
  GOTOXY(m_bMSG_ROW, m_bMSG_COL);
  CLREOL;
  WRITE(i_stMsg);
END;
{========================================================= eoP View_Prompt() }

{============================================================= View_GetCmd() }
{ Purpose...: Print command prompt on bottom line and get user response
{ Input.....: i_fMore.: TRUE= there is more to come, <F>orward shown
{             i_fFirst: TRUE= first screen shown, <B>ackward not shown
{ Result....: Entered character
{============================================================================}
FUNCTION View_GetCmd(i_fFirst: BOOLEAN; i_fMore: BOOLEAN): CHAR;

  VAR stKeys : STRING[9];
  VAR stMsg  : g_tst;

BEGIN
  stKeys := g_cESC;
  stMsg := m_stFile + ': ';

  IF (i_fMore = TRUE) THEN BEGIN
    stMsg := stMsg + '<F>wd | ';
    stKeys := stKeys + 'Ff';
  END;

  IF (i_fFirst = FALSE) THEN BEGIN
    stMsg := stMsg + '<B>ack | ';
    stKeys := stKeys + 'Bb';
  END;

  stMsg := stMsg + 'Print f<i>le or <s>creen | <esc> exit ';
  stKeys := stKeys + 'IiSs';

  View_Prompt(stMsg);
  
  View_GetCmd := UPCASE(Lib_GetKey(stKeys, TRUE));
END;
{========================================================= eoF View_GetCmd() }

{========================================================== View_PrintFile() }
{ Purpose...: Print text file. Put in an own procedure because we have to
{             open the file in binary mode for ASM listings to print 
{             correctly. Those files contain lines with only CR and no LF
{             as sent by the TM990 assembler.
{============================================================================}
PROCEDURE View_PrintFile;
 
  TYPE tabBuf = ARRAY[1..128] OF BYTE;
  
  VAR oFile     : FILE;
  VAR stLine    : g_tst;
  VAR abBuf     : tabBuf;
  VAR iRecsRead : INTEGER;    
  VAR bNdx      : BYTE;
  VAR fDone     : BOOLEAN;
  VAR cCh       : CHAR;
    
BEGIN
  {--------------------------------------- make sure the text file is closed }
  File_CloseFile;

  {-------------------------------------------- open the file in binary mode }  
  ASSIGN(oFile, m_stFile);
  {$I-} RESET(oFile) {$I+};
  
  IF (IORESULT = 0) THEN BEGIN
    fDone := FALSE;
    stLine := '';
        
    WHILE ((NOT EOF(oFile)) AND (fDone = FALSE)) DO BEGIN
      BLOCKREAD(oFile, abBuf, 1, iRecsRead);
      
      IF (iRecsRead = 1) THEN BEGIN
        bNdx := 1;

        WHILE ((bNdx <= 128) AND (fDone = FALSE)) DO BEGIN
          cCh := CHR(abBuf[bNdx]);
          bNdx := bNdx + 1;
                    
          IF ((cCh <> g_cLF) AND (cCh <> g_cEOF)) THEN BEGIN
            stLine := stLine + cCh;
          END;

          IF (cCh = g_cLF) THEN BEGIN
            fDone := NOT LibPrt_Print(CHR(0), stLine + g_cLF);
            stLine := '';
          END ELSE BEGIN
            fDone := (cCh = g_cEOF);
          END;  

          IF (fDone = FALSE) THEN BEGIN
            fDone := (Lib_GetKey(g_cESC, FALSE) = g_cESC);
          END;  
        END;
      END;      
    END;

    CLOSE(oFile);
  END;
END;
{====================================================== eoF View_PrintFile() }

{==================================================================== View() }
{ Purpose...: View text file
{============================================================================}
PROCEDURE View;

  VAR pstLine       : g_tpstLine;
  VAR pfEof         : ^BOOLEAN;
  VAR fEof          : BOOLEAN;
  VAR iBufLine      : INTEGER;
  VAR iFirstLine    : INTEGER;
  VAR iFirstLineBak : INTEGER;
  VAR iScrLines     : INTEGER;
  VAR iNumLines     : INTEGER;
  VAR fPrintScreen  : BOOLEAN;
  VAR fPrintFile    : BOOLEAN;
  VAR fFilePrinted  : BOOLEAN;
  VAR fPrtTimeout   : BOOLEAN;
  VAR fStart        : BOOLEAN;
  VAR fExit         : BOOLEAN;
  VAR cKbd          : CHAR;

BEGIN
  pstLine := NIL;
  pfEof := PTR(ADDR(fEof));
  fEof := FALSE;
  iBufLine := 0;
  iFirstLine := 0;
  iFirstLineBak := 0;
  iScrLines := 0;
  fPrintScreen := FALSE;
  fFilePrinted := FALSE;
  fPrtTimeout := FALSE;
  fStart := TRUE;
  fExit := FALSE;
  cKbd := CHR(0);
  iNumLines := File_GetBufLines;

  {-------------------------- set top-of-form in case screen prints are done }
  LibPrt_Setup(m_bMSG_ROW, m_bMSG_COL, TRUE);

  {--------------------------- need to save filename as print file closes it }
  m_stFile := g_stFile;  

  {------------------------------------------------ loop until exit selected }
  WHILE (fExit = FALSE) DO BEGIN
    fPrtTimeout := FALSE;
    IF ((fPrintScreen = FALSE) AND (fFilePrinted = FALSE)) THEN CLRSCR;
    pstLine := PTR(1);
    iBufLine := iFirstLine;
    iScrLines := 0;
    fEof := FALSE;

    {---------------------------------------------------- process one screen }
    WHILE ((pstLine <> NIL) AND 
           (iScrLines < m_bSCR_LINES) AND 
           (fFilePrinted = FALSE)) DO BEGIN
           
      pstLine := File_GetString(iBufLine, fStart, pfEof);
      fStart := FALSE;
      iBufLine := -1;
      fEof := pfEof^;

      {----------------------------------------- continue if not end of file }
      IF (pstLine <> NIL) THEN BEGIN

        {----------------------------------------- print or display one line }
        IF (fPrintScreen = TRUE) THEN BEGIN
          fPrtTimeout := NOT LibPrt_Print(CHR(0), pstLine^ + g_stCRLF);
        END ELSE BEGIN
          WRITELN(COPY(pstLine^, 1, 79));
        END;

        iScrLines := iScrLines + 1;

        {------------------------------- allow printing to abort using <esc> }
        IF (fPrintScreen = TRUE) THEN BEGIN
          IF (Lib_GetKey(g_cESC, FALSE) = g_cESC) THEN BEGIN
            iScrLines := m_bSCR_LINES;
          END;
        END;

        IF (fPrtTimeout = TRUE) THEN BEGIN
          iScrLines := m_bSCR_LINES;
        END;
      END;
    END;

    {------------------------------------------------ cleanup after printing }
    fFilePrinted := FALSE;
    
    IF (fPrintScreen = TRUE) THEN BEGIN
      fPrintScreen := FALSE;
      iFirstLine := iFirstLineBak;
    END;

    {------------------------------------------------ read command from user }
    cKbd := View_GetCmd((iFirstLine = 0), (fEof = FALSE));

    {--------------------------------------------------- display next screen }
    IF (cKbd = 'F') THEN BEGIN
      iFirstLine := iFirstLine + m_bSCR_LINES;
    END;

    {----------------------------------------------- display previous screen }
    IF (cKbd = 'B') THEN BEGIN
      iFirstLine := iFirstLine - m_bSCR_LINES;
      fStart := TRUE;
    END;

    {------------------------------------------------------ print whole file }
    IF (cKbd = 'I') THEN BEGIN
      View_Prompt(m_stFile + ': Printing file' + g_stMSG_CANCEL);
      LibPrt_Setup(m_bMSG_ROW, m_bMSG_COL, TRUE);
      View_PrintFile;
      fFilePrinted := TRUE;
      LibPrt_Setup(m_bMSG_ROW, m_bMSG_COL, TRUE);
    END;

    {------------------------------------ print actual screen, backup values }
    IF (cKbd = 'S') THEN BEGIN
      View_Prompt(m_stFile + ': Printing screen' + g_stMSG_CANCEL);
      iFirstLineBak := iFirstLine;
      fPrintScreen := TRUE;
      fStart := TRUE;
    END;

    {-------------------------------------- exit, free buffer and close file }
    IF (cKbd = g_cESC) THEN BEGIN
      File_CloseFile;
      CLRSCR;
      fExit := TRUE;
    END;
  END;
END;
{================================================================ eoP View() }

{########################################################## EOF TM990VIW.INC }